/***{{{*******************************************************************
 * This file is part of libco - object library for C                     *
 * Copyright (c) 2002                                                    *
 *     Peter Rockai (yenar) <yenar@host.sk>                              *
 *                                                                       *
 * This library is free software; you can redistribute it and/or         *
 * modify it under the terms of the GNU Lesser General Public            *
 * License as published by the Free Software Foundation; either          *
 * version 2 of the License, or (at your option) any later version.      *
 *                                                                       *
 * This library is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 * Lesser General Public License for more details.                       *
 *                                                                       *
 * You should have received a copy of the GNU Lesser General Public      *
 * License along with this library; if not, write to the Free Software   *
 * Foundation, Inc., 59 Temple Place, Suite 330,                         *
 * Boston, MA 02111-1307  USA                                            *
 *******************************************************************}}}***/
/* {{{ file description */
/** \file co_tab.c
 *  \brief Generic table object implementation. */
/* }}} */

/* {{{ includes */
#include <stdarg.h>
#include <string.h>

#include "co_tab.h"
#include "co_vector.h"
#include "co_util.h"
#include "co_debug.h"
/* }}} */

#include "co_tab.cometa"

/* {{{ */
co_tab co_tab_new (co hdr)
	/// Create new table and initialize all members to meaningful values.
	// FIXME: co_tab is in unusable state :-/
	/** \return newly created co_table on success, 0 on failure */
{
	co_tab tab;
	co vec;

	vec = CO_NEW (co_vector);

	tab = new (co_tab_s);
	tab -> vec = vec;
	tab -> col = 0;
	tab -> hdr = hdr;
	co_tab_meth_ar_init (tab);
	cox_pop (vec);

	CO_BLESS (tab, co_tab, 0);
	cox_pop (tab);

	return tab;
}
/* }}} */
/* {{{ */
CO_METH (co_tab, item_get)
{
	co _i;
	CO_METH_INI (co_tab, item_get, int i, int j);
	_i = CO_M (this -> vec, item_get, i) . _co;
	return CO_M (_i, item_get, j);
}
/* }}} */

/* {{{ */
co co_field_set_new (void)
{
	co_field_set fs;

	fs = new (co_field_set_s);
	fs -> d = CO_NEW (co_vector);
	co_field_set_meth_ar_init (fs);
	cox_pop (fs -> d);

	CO_BLESS (fs, co_field_set, 0);
	cox_pop (fs);

	return fs;
}
/* }}} */
/* {{{ */
CO_METH (co_field_set, item_get)
{
	CO_METH_INI (co_field_set, item_get, int i);
	CO_RETURN (co, CO_M (this -> d, item_get, i) . _co);
}
/* }}} */

/* {{{ */
static co_v co_field_msg (co o, co s, co_id_t a, va_list ap)
{
	co_field f = o;
	co_v rv;
	(void) ap;
	(void) s;
	rv . _int = 0;
	CO_MH_BEGIN;
	CO_MH_CASE (a, destroy) {
		free (f);
	}
	return rv;
}
/* }}} */
/* {{{ */
co co_field_new (const char *name, co_type t, int w, int flags)
{
	co_field f;

	f = new (co_field_s);
	f -> name = name;
	f -> width = w;
	f -> t = t;
	f -> flags = flags;

	CO_BLESS_M (f, co_field);
	cox_pop (f);

	return f;
}
/* }}} */

#if 0
/* {{{ static prototypes */
static co_v co_tab_msg (co o, co_id_t a, va_list ap);
static co_v co_field_set_msg (co o, co_id_t a, va_list ap);
static co_vr co_field_p (const co o, co_id_t p);
static co_vr co_field_set_p (const co o, co_id_t p);
/* }}} */
/* {{{ */ co_v co_tab_msg (co o, co_id_t a, va_list ap)
	/// Handle incoming message for table.
{
	co_tab t = o;
	co_v rv;
	va_list ap2;

	__va_copy (ap2, ap);
	rv . i = 0xBAD0;

	CO_MH_BEGIN;
	CO_MH_CASE (a, item_get) {
		int i = va_arg (ap, int);
		int j = va_arg (ap, int);
		co _i;
		_i = CO_M (t -> vec, item_get, i) . o;
		// shall return co, probably with implemented strval
		rv = CO_M (_i, item_get, j);
	}
	/* CO_MH_CASE (a, item_str_get) {
		int i = va_arg (ap, int);
		int j = va_arg (ap, int);
		co _i;
		_i = CO_M (t -> vec, item_get, i) . o;
		// shall return char *, strval of j-th column
		rv = CO_M (_i, item_str_get, j);
	} */
	CO_MH_CASE (a, destroy) {
		co_del (t -> vec);
		free (t);
	}
	CO_MH_UNHANDLED (a);
	return rv;
} /* }}} */
/* {{{ */ co_vr co_field_p (const co o, co_id_t p)
{
	co_field f = o;
	CO_PH_BEGIN;
	CO_PH_RET_C (p, name, c_p, f -> name);
	CO_PH_RET (p, width, i, f -> width);
	CO_PH_RET (p, type, t, f -> t);
	CO_PH_RET (p, flags, i, f -> flags);
	CO_PH_DEFAULT (p);
} /* }}} */
/* {{{ */ co_vr co_field_set_p (const co o, co_id_t p)
{
	co_field_set fs = o;
	CO_PH_BEGIN;
	CO_PH_RET (p, nitems, i, fs -> d -> items);
	CO_PH_DEFAULT (p);
} /* }}} */
/* {{{ */ co_v co_field_set_msg (co o, co_id_t a, va_list ap)
{
	co_field_set fs = o;
	co_v rv;

	rv . i = 0xBAD0;

	CO_MH_BEGIN;
	CO_MH_CASE (a, item_get) {
		int i = va_arg (ap, int);
		rv . o = CO_M (fs -> d, item_get, i) . o;
	}
	CO_MH_CASE (a, item_add) {
		co _o = va_arg (ap, co);
		CO_M (fs -> d, item_add, _o);
		cox_pop (_o);
	}
	CO_MH_CASE (a, destroy) {
		co_del (fs -> d);
		free (fs);
	}
	CO_MH_UNHANDLED (a);
	return rv;
} /* }}} */
#endif /* 0 */
